# Generated by Django 2.1.5 on 2019-02-09 19:33
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_celery_beat', '0006_periodictask_priority'),
    ]

    operations = [
        migrations.AddField(
            model_name='periodictask',
            name='headers',
            field=models.TextField(
                blank=True,
                default='{}',
                help_text='JSON encoded message headers',
                verbose_name='Message headers'
            ),
        ),
    ]
