# Generated by Django 5.0.1 on 2024-07-04 07:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_celery_beat', '0018_improve_crontab_helptext'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='periodictasks',
            options={'verbose_name': 'periodic task track', 'verbose_name_plural': 'periodic task tracks'},
        ),
    ]
