# Generated by Django 1.9.11 on 2016-12-09 00:49
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_celery_beat', '0002_auto_20161118_0346'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='solarschedule',
            options={
                'ordering': ('event', 'latitude', 'longitude'),
                'verbose_name': 'solar event',
                'verbose_name_plural': 'solar events'},
        ),
        migrations.AlterUniqueTogether(
            name='solarschedule',
            unique_together=set([('event', 'latitude', 'longitude')]),
        ),
    ]
